﻿using System;
using System.Collections.Generic;

public abstract class TupleComparerBase {

    private int[] m_RankOrder;

    protected TupleComparerBase(params int[] rankOrder){
        m_RankOrder = rankOrder;
        foreach(int rank in m_RankOrder){
            int absRank = Math.Abs(rank);
            if(absRank < 0 || absRank > m_RankOrder.Length){
                throw new IndexOutOfRangeException();
            }
        }
    }

    public int[] RankOrder{
        get{
            return m_RankOrder;
        }
    }


} // class TupleComparerBase

public class TupleComparer <T1> : TupleComparerBase, IComparer<ITuple<T1>>
        where T1 : IComparable<T1>
{

    public TupleComparer() : base(1) {
    }

    public TupleComparer(int rank1) : base(rank1) {
    }

    /* private implement */ int IComparer<ITuple<T1>>.Compare(ITuple<T1> a, ITuple<T1> b) {
        int ret = 0;
        foreach(int rank in this.RankOrder){
            int absRank = Math.Abs(rank);
            switch(absRank){
                case 1:
                    ret = a.Value1.CompareTo(b.Value1);
                    break;
            }
            if(rank < 0){
                ret *= -1;
            }
            if(ret != 0){
                return ret;
            }
        }
        return 0;        
    }

	///* private implement */ bool IComparer<ITuple<T1>>.Equals(ITuple<T1> a, ITuple<T1> b) {
	//    return 0 == ((IComparer<ITuple<T1>>)this).Compare(a, b);
	//}

	///* private implement */ int IComparer<ITuple<T1>>.GetHashCode(ITuple<T1> obj) {
	//    return obj.GetHashCode();
	//}

} // class TupleComparer<T1>

public class TupleComparer <T1, T2> : TupleComparerBase, IComparer<ITuple<T1, T2>>
        where T1 : IComparable<T1> where T2 : IComparable<T2>
{

    public TupleComparer() : base(1, 2) {
    }

    public TupleComparer(int rank1, int rank2) : base(rank1, rank2) {
    }

    /* private implement */ int IComparer<ITuple<T1, T2>>.Compare(ITuple<T1, T2> a, ITuple<T1, T2> b) {
        int ret = 0;
        foreach(int rank in this.RankOrder){
            int absRank = Math.Abs(rank);
            switch(absRank){
                case 1:
                    ret = a.Value1.CompareTo(b.Value1);
                    break;
                case 2:
                    ret = a.Value2.CompareTo(b.Value2); 
                    break;
            }
            if(rank < 0){
                ret *= -1;
            }
            if(ret != 0){
                return ret;
            }
        }
        return 0;        
    }

	///* private implement */ bool IComparer<ITuple<T1, T2>>.Equals(ITuple<T1, T2> a, ITuple<T1, T2> b) {
	//    return 0 == ((IComparer<ITuple<T1, T2>>)this).Compare(a, b);
	//}

	///* private implement */ int IComparer<ITuple<T1, T2>>.GetHashCode(ITuple<T1, T2> obj) {
	//    return obj.GetHashCode();
	//}

} // class TupleComparer<T1, T2>

public class TupleComparer <T1, T2, T3> : TupleComparerBase, IComparer<ITuple<T1, T2, T3>>
        where T1 : IComparable<T1> where T2 : IComparable<T2> where T3 : IComparable<T3>
{

    public TupleComparer() : base(1, 2, 3) {
    }

    public TupleComparer(int rank1, int rank2, int rank3) : base(rank1, rank2, rank3) {
    }

    /* private implement */ int IComparer<ITuple<T1, T2, T3>>.Compare(ITuple<T1, T2, T3> a, ITuple<T1, T2, T3> b) {
        int ret = 0;
        foreach(int rank in this.RankOrder){
            int absRank = Math.Abs(rank);
            switch(absRank){
                case 1:
                    ret = a.Value1.CompareTo(b.Value1);
                    break;
                case 2:
                    ret = a.Value2.CompareTo(b.Value2); 
                    break;
                case 3:
                    ret = a.Value3.CompareTo(b.Value3);
                    break;
            }
            if(rank < 0){
                ret *= -1;
            }
            if(ret != 0){
                return ret;
            }
        }
        return 0;        
    }

	///* private implement */ bool IComparer<ITuple<T1, T2, T3>>.Equals(ITuple<T1, T2, T3> a, ITuple<T1, T2, T3> b) {
	//    return 0 == ((IComparer<ITuple<T1, T2, T3>>)this).Compare(a, b);
	//}

	///* private implement */ int IComparer<ITuple<T1, T2, T3>>.GetHashCode(ITuple<T1, T2, T3> obj) {
	//    return obj.GetHashCode();
	//}

} // class TupleComparer<T1, T2, T3>

public class TupleComparer <T1, T2, T3, T4> : TupleComparerBase, IComparer<ITuple<T1, T2, T3, T4>>
        where T1 : IComparable<T1> where T2 : IComparable<T2> where T3 : IComparable<T3> where T4 : IComparable<T4>
{

    public TupleComparer() : base(1, 2, 3, 4) {
    }

    public TupleComparer(int rank1, int rank2, int rank3, int rank4) : base(rank1, rank2, rank3, rank4) {
    }

    /* private implement */ int IComparer<ITuple<T1, T2, T3, T4>>.Compare(ITuple<T1, T2, T3, T4> a, ITuple<T1, T2, T3, T4> b) {
        int ret = 0;
        foreach(int rank in this.RankOrder){
            int absRank = Math.Abs(rank);
            switch(absRank){
                case 1:
                    ret = a.Value1.CompareTo(b.Value1);
                    break;
                case 2:
                    ret = a.Value2.CompareTo(b.Value2); 
                    break;
                case 3:
                    ret = a.Value3.CompareTo(b.Value3);
                    break;
                case 4:
                    ret = a.Value4.CompareTo(b.Value4);
                    break;
            }
            if(rank < 0){
                ret *= -1;
            }
            if(ret != 0){
                return ret;
            }
        }
        return 0;        
    }

	///* private implement */ bool IComparer<ITuple<T1, T2, T3, T4>>.Equals(ITuple<T1, T2, T3, T4> a, ITuple<T1, T2, T3, T4> b) {
	//    return 0 == ((IComparer<ITuple<T1, T2, T3, T4>>)this).Compare(a, b);
	//}

	///* private implement */ int IComparer<ITuple<T1, T2, T3, T4>>.GetHashCode(ITuple<T1, T2, T3, T4> obj) {
	//    return obj.GetHashCode();
	//}

} // class TupleComparer<T1, T2, T3, T4>

public class TupleComparer <T1, T2, T3, T4, T5> : TupleComparerBase, IComparer<ITuple<T1, T2, T3, T4, T5>>
        where T1 : IComparable<T1> where T2 : IComparable<T2> where T3 : IComparable<T3> where T4 : IComparable<T4> where T5 : IComparable<T5>
{

    public TupleComparer() : base(1, 2, 3, 4, 5) {
    }

    public TupleComparer(int rank1, int rank2, int rank3, int rank4, int rank5) : base(rank1, rank2, rank3, rank4, rank5) {
    }

    /* private implement */ int IComparer<ITuple<T1, T2, T3, T4, T5>>.Compare(ITuple<T1, T2, T3, T4, T5> a, ITuple<T1, T2, T3, T4, T5> b) {
        int ret = 0;
        foreach(int rank in this.RankOrder){
            int absRank = Math.Abs(rank);
            switch(absRank){
                case 1:
                    ret = a.Value1.CompareTo(b.Value1);
                    break;
                case 2:
                    ret = a.Value2.CompareTo(b.Value2); 
                    break;
                case 3:
                    ret = a.Value3.CompareTo(b.Value3);
                    break;
                case 4:
                    ret = a.Value4.CompareTo(b.Value4);
                    break;
                case 5:
                    ret = a.Value5.CompareTo(b.Value5);
                    break;
            }
            if(rank < 0){
                ret *= -1;
            }
            if(ret != 0){
                return ret;
            }
        }
        return 0;        
    }

	///* private implement */ bool IComparer<ITuple<T1, T2, T3, T4, T5>>.Equals(ITuple<T1, T2, T3, T4, T5> a, ITuple<T1, T2, T3, T4, T5> b) {
	//    return 0 == ((IComparer<ITuple<T1, T2, T3, T4, T5>>)this).Compare(a, b);
	//}
    
	///* private implement */ int IComparer<ITuple<T1, T2, T3, T4, T5>>.GetHashCode(ITuple<T1, T2, T3, T4, T5> obj) {
	//    return obj.GetHashCode();
	//}

} // class TupleComparer<T1, T2, T3, T4, T5>
