using System;

public abstract class TupleBase : IRandomAccessTuple, IArrayAccessTuple {

    public abstract int ElementCount {get;}
    public abstract object GetElement(int index);
    public abstract object[] GetElements();
    public abstract void SetElement(int index, object value);
    public abstract void SetElements(params object[] values);

} // abstract class TupleBase

public class Tuple<T1> : TupleBase, ITuple<T1> {

    private T1 m_Value1;

    public Tuple(T1 v1){
        m_Value1 = v1;
    }

    public T1 Value1 {
        get {
            return m_Value1;
        }
        set{
            m_Value1 = value;
        }
    }

    public override int ElementCount {
        get {
            return 1;
        }
    }

    public override string ToString() {
        return "" + m_Value1;
    }

    public override bool Equals(object obj) {
        if(obj != null && obj.GetType().Equals(this.GetType())) {
            Tuple<T1> that = (Tuple<T1>) obj;
            return object.Equals(this.m_Value1, that.m_Value1);
        } else {
            return false;
        }
    }

    public override int GetHashCode() {
        return (m_Value1 != null ? m_Value1.GetHashCode() : 0);
    }

    public override object GetElement(int index) {
        switch(index) {
            case 0:
                return m_Value1;
            default:
                throw new IndexOutOfRangeException();
        }
    }

    public override void SetElement(int index, object value){
        switch(index){
            case 0:
                m_Value1 = (T1) value;
                break;
            default:
                throw new IndexOutOfRangeException();
        }
    }

    public override object[] GetElements() {
        return new object[] {m_Value1};
    }

    public override void SetElements(params object[] values){
        if(values.Length > 0){
            m_Value1 = (T1) values[0];
        }
    }

} // class Tuple<T1>

public class Tuple <T1, T2> : Tuple<T1>, ITuple<T1, T2> {

    private T2 m_Value2;

    public Tuple(T1 v1, T2 v2) : base(v1) {
        m_Value2 = v2;
    }

    public T2 Value2 {
        get {
            return m_Value2;
        }
         set{
            m_Value2 = value;
        }
   }

    public override int ElementCount{
        get{
            return 2;
        }
    }

    public override string ToString() {
        return base.ToString() + "; " + m_Value2;
    }

    public override bool Equals(object obj) {
        if(obj != null && obj.GetType().Equals(this.GetType())){
            Tuple<T1, T2> that = (Tuple<T1, T2>) obj;
            return base.Equals(obj)
                && object.Equals(this.m_Value2, that.m_Value2);
        }else{
            return false;
        }
    }

    public override int GetHashCode() {
        int hcBase = base.GetHashCode();
        int hcThis = (m_Value2 != null ? m_Value2.GetHashCode() : 0);
        return hcBase ^ hcThis;
    }

    public override object GetElement(int index){
        switch(index){
            case 1:
                return m_Value2;
            default:
                return base.GetElement(index);
        }
    }

    public override void SetElement(int index, object value){
        switch(index){
            case 1:
                m_Value2 = (T2) value;
                break;
            default:
                base.SetElement(index, value);
                break;
        }
    }

    public override object[] GetElements() {
        return new object[] {this.Value1, m_Value2};
    }

    public override void SetElements(params object[] values){
        base.SetElements(values);
        if(values.Length > 1){
            m_Value2 = (T2) values[1];
        }
    }

} // class Tuple<T1, T2>

public class Tuple <T1, T2, T3> : Tuple<T1, T2>, ITuple<T1, T2, T3> {

    private T3 m_Value3;

    public Tuple(T1 v1, T2 v2, T3 v3) : base(v1, v2) {
        m_Value3 = v3;
    }

    public T3 Value3{
        get{
            return m_Value3;
        }
        set{
            m_Value3 = value;
        }
    }

    public override int ElementCount {
        get {
            return 3;
        }
    }

    public override string ToString() {
        return base.ToString() + "; " + m_Value3;
    }

    public override bool Equals(object obj) {
        if(obj != null && obj.GetType().Equals(this.GetType())) {
            Tuple<T1, T2, T3> that = (Tuple<T1, T2, T3>) obj;
            return base.Equals(obj)
                && object.Equals(this.m_Value3, that.m_Value3);
        } else {
            return false;
        }
    }

    public override int GetHashCode() {
        int hcBase = base.GetHashCode();
        int hcThis = (m_Value3 != null ? m_Value3.GetHashCode() : 0);
        return hcBase ^ hcThis;
    }

    public override object GetElement(int index) {
        switch(index) {
            case 2:
                return m_Value3;
            default:
                return base.GetElement(index);
        }
    }

    public override void SetElement(int index, object value){
        switch(index){
            case 2:
                m_Value3 = (T3) value;
                break;
            default:
                base.SetElement(index, value);
                break;
        }
    }

    public override object[] GetElements() {
        return new object[] {
            this.Value1, this.Value2, m_Value3
        };
    }

    public override void SetElements(params object[] values){
        base.SetElements(values);
        if(values.Length > 2){
            m_Value3 = (T3) values[2];
        }
    }

} // class Tuple<T1, T2, T3>

public class Tuple<T1, T2, T3, T4> : Tuple<T1, T2, T3>, ITuple<T1, T2, T3, T4> {

    private T4 m_Value4;

    public Tuple(T1 v1, T2 v2, T3 v3, T4 v4) : base(v1, v2, v3) {
        m_Value4 = v4;
    }

    public T4 Value4 {
        get {
            return m_Value4;
        }
        set{
            m_Value4 = value;
        }
    }

    public override int ElementCount {
        get {
            return 4;
        }
    }

    public override string ToString() {
        return base.ToString() + "; " + m_Value4;
    }

    public override bool Equals(object obj) {
        if(obj != null && obj.GetType().Equals(this.GetType())) {
            Tuple<T1, T2, T3, T4> that = (Tuple<T1, T2, T3, T4>)obj;
            return base.Equals(obj)
                && object.Equals(this.m_Value4, that.m_Value4);
        } else {
            return false;
        }
    }

    public override int GetHashCode() {
        int hcBase = base.GetHashCode();
        int hcThis = (m_Value4 != null ? m_Value4.GetHashCode() : 0);
        return hcBase ^ hcThis;
    }

    public override object GetElement(int index) {
        switch(index) {
            case 3:
                return m_Value4;
            default:
                return base.GetElement(index);
        }
    }

    public override void SetElement(int index, object value){
        switch(index){
            case 3:
                m_Value4 = (T4) value;
                break;
            default:
                base.SetElement(index, value);
                break;
        }
    }

    public override object[] GetElements() {
        return new object[] {
            this.Value1, this.Value2, this.Value3, m_Value4
        };
    }

    public override void SetElements(params object[] values){
        base.SetElements(values);
        if(values.Length > 3){
            m_Value4 = (T4) values[3];
        }
    }

} // class Tuple<T1, T2, T3, T4>

public class Tuple<T1, T2, T3, T4, T5> : Tuple<T1, T2, T3, T4>, ITuple<T1, T2, T3, T4, T5> {

    private T5 m_Value5;

    public Tuple(T1 v1, T2 v2, T3 v3, T4 v4, T5 v5) : base(v1, v2, v3, v4) {
        m_Value5 = v5;
    }

    public T5 Value5 {
        get {
            return m_Value5;
        }
        set{
            m_Value5 = value;
        }
    }

    public override int ElementCount {
        get {
            return 5;
        }
    }

    public override string ToString() {
        return base.ToString() + "; " + m_Value5;
    }

    public override bool Equals(object obj) {
        if(obj != null && obj.GetType().Equals(this.GetType())) {
            Tuple<T1, T2, T3, T4, T5> that = (Tuple<T1, T2, T3, T4, T5>)obj;
            return base.Equals(obj)
                && object.Equals(this.m_Value5, that.m_Value5);
        } else {
            return false;
        }
    }

    public override int GetHashCode() {
        int hcBase = base.GetHashCode();
        int hcThis = (m_Value5 != null ? m_Value5.GetHashCode() : 0);
        return hcBase ^ hcThis;
    }

    public override object GetElement(int index) {
        switch(index) {
            case 4:
                return m_Value5;
            default:
                return base.GetElement(index);
        }
    }

    public override void SetElement(int index, object value){
        switch(index){
            case 4:
                m_Value5 = (T5) value;
                break;
            default:
                base.SetElement(index, value);
                break;
        }
    }

    public override object[] GetElements() {
        return new object[] {
            this.Value1, this.Value2, this.Value3, this.Value4, m_Value5
        };
    }

    public override void SetElements(params object[] values){
        base.SetElements(values);
        if(values.Length > 4){
            m_Value5 = (T5) values[4];
        }
    }

} // class Tuple<T1, T2, T3, T4, T5>
