﻿using System;
using System.Collections.Generic;

public delegate bool TupleMatcher<T1>(ITuple<T1> a, ITuple<T1> b);
public delegate bool TupleMatcher<T1, T2>(ITuple<T1, T2> a, ITuple<T1, T2> b);
public delegate bool TupleMatcher<T1, T2, T3>(ITuple<T1, T2, T3> a, ITuple<T1, T2, T3> b);
public delegate bool TupleMatcher<T1, T2, T3, T4>(ITuple<T1, T2, T3, T4> a, ITuple<T1, T2, T3, T4> b);
public delegate bool TupleMatcher<T1, T2, T3, T4, T5>(ITuple<T1, T2, T3, T4, T5> a, ITuple<T1, T2, T3, T4, T5> b);

public static class TupleUtil {

    public static IEnumerable<ITuple<T1>> SelectTuples<T1>(IEnumerable<ITuple<T1>> source, T1 match1){
        ITuple<T1> match = new Tuple<T1>(match1);
        return TupleUtil.SelectTuples<T1>(source, match, delegate(ITuple<T1> a, ITuple<T1> b){
            return (a.Value1 == null || a.Value1.Equals(default(T1)) || a.Value1.Equals(b.Value1)); // ... value type default match always succeeds!
        });
    }

    public static IEnumerable<ITuple<T1, T2>> SelectTuples<T1, T2>(IEnumerable<ITuple<T1, T2>> source, T1 match1, T2 match2){
        ITuple<T1, T2> match = new Tuple<T1, T2>(match1, match2);
        return TupleUtil.SelectTuples<T1, T2>(source, match, delegate(ITuple<T1, T2> a, ITuple<T1, T2> b){
            return (a.Value1 == null || a.Value1.Equals(default(T1)) || a.Value1.Equals(b.Value1))
                && (a.Value2 == null || a.Value2.Equals(default(T2)) || a.Value2.Equals(b.Value2)); // ... value type default match always succeeds!
        });
    }

    public static IEnumerable<ITuple<T1, T2, T3>> SelectTuples<T1, T2, T3>(IEnumerable<ITuple<T1, T2, T3>> source, T1 match1, T2 match2, T3 match3){
        ITuple<T1, T2, T3> match = new Tuple<T1, T2, T3>(match1, match2, match3);
        return TupleUtil.SelectTuples<T1, T2, T3>(source, match, delegate(ITuple<T1, T2, T3> a, ITuple<T1, T2, T3> b){
            return (a.Value1 == null || a.Value1.Equals(default(T1)) || a.Value1.Equals(b.Value1))
                && (a.Value2 == null || a.Value2.Equals(default(T2)) || a.Value2.Equals(b.Value2))
                && (a.Value3 == null || a.Value3.Equals(default(T3)) || a.Value3.Equals(b.Value3)); // ... value type default match always succeeds!
        });
    }

    public static IEnumerable<ITuple<T1, T2, T3, T4>> SelectTuples<T1, T2, T3, T4>(IEnumerable<ITuple<T1, T2, T3, T4>> source, T1 match1, T2 match2, T3 match3, T4 match4){
        ITuple<T1, T2, T3, T4> match = new Tuple<T1, T2, T3, T4>(match1, match2, match3, match4);
        return TupleUtil.SelectTuples<T1, T2, T3, T4>(source, match, delegate(ITuple<T1, T2, T3, T4> a, ITuple<T1, T2, T3, T4> b){
            return (a.Value1 == null || a.Value1.Equals(default(T1)) || a.Value1.Equals(b.Value1))
                && (a.Value2 == null || a.Value2.Equals(default(T2)) || a.Value2.Equals(b.Value2))
                && (a.Value3 == null || a.Value3.Equals(default(T3)) || a.Value3.Equals(b.Value3))
                && (a.Value4 == null || a.Value4.Equals(default(T4)) || a.Value4.Equals(b.Value4)); // ... value type default match always succeeds!
        });
    }

    public static IEnumerable<ITuple<T1, T2, T3, T4, T5>> SelectTuples<T1, T2, T3, T4, T5>(IEnumerable<ITuple<T1, T2, T3, T4, T5>> source, T1 match1, T2 match2, T3 match3, T4 match4, T5 match5){
        ITuple<T1, T2, T3, T4, T5> match = new Tuple<T1, T2, T3, T4, T5>(match1, match2, match3, match4, match5);
        return TupleUtil.SelectTuples<T1, T2, T3, T4, T5>(source, match, delegate(ITuple<T1, T2, T3, T4, T5> a, ITuple<T1, T2, T3, T4, T5> b){
            return (a.Value1 == null || a.Value1.Equals(default(T1)) || a.Value1.Equals(b.Value1))
                && (a.Value2 == null || a.Value2.Equals(default(T2)) || a.Value2.Equals(b.Value2))
                && (a.Value3 == null || a.Value3.Equals(default(T3)) || a.Value3.Equals(b.Value3))
                && (a.Value4 == null || a.Value4.Equals(default(T4)) || a.Value4.Equals(b.Value4))
                && (a.Value5 == null || a.Value5.Equals(default(T5)) || a.Value5.Equals(b.Value5)); // ... value type default match always succeeds!
        });
    }

    public static IEnumerable<ITuple<T1>> SelectTuples<T1>(IEnumerable<ITuple<T1>> source, ITuple<T1> match, TupleMatcher<T1> matchCallback){
        foreach(ITuple<T1> t in source){
            if(matchCallback(match, t)){
                yield return t;
            }
        }
    }

    public static IEnumerable<ITuple<T1, T2>> SelectTuples<T1, T2>(IEnumerable<ITuple<T1, T2>> source, ITuple<T1, T2> match, TupleMatcher<T1, T2> matchCallback){
        foreach(ITuple<T1, T2> t in source){
            if(matchCallback(match, t)){
                yield return t;
            }
        }
    }

    public static IEnumerable<ITuple<T1, T2, T3>> SelectTuples<T1, T2, T3>(IEnumerable<ITuple<T1, T2, T3>> source, ITuple<T1, T2, T3> match, TupleMatcher<T1, T2, T3> matchCallback){
        foreach(ITuple<T1, T2, T3> t in source){
            if(matchCallback(match, t)){
                yield return t;
            }
        }
    }

    public static IEnumerable<ITuple<T1, T2, T3, T4>> SelectTuples<T1, T2, T3, T4>(IEnumerable<ITuple<T1, T2, T3, T4>> source, ITuple<T1, T2, T3, T4> match, TupleMatcher<T1, T2, T3, T4> matchCallback){
        foreach(ITuple<T1, T2, T3, T4> t in source){
            if(matchCallback(match, t)){
                yield return t;
            }
        }
    }

    public static IEnumerable<ITuple<T1, T2, T3, T4, T5>> SelectTuples<T1, T2, T3, T4, T5>(IEnumerable<ITuple<T1, T2, T3, T4, T5>> source, ITuple<T1, T2, T3, T4, T5> match, TupleMatcher<T1, T2, T3, T4, T5> matchCallback){
        foreach(ITuple<T1, T2, T3, T4, T5> t in source){
            if(matchCallback(match, t)){
                yield return t;
            }
        }
    }

}
